︠a05781db-d2b7-43c6-8b42-175e802fc00ci︠
%html
In order to look at crystals, we first start with the Lie algebra $\mathfrak{sl}_2 = \{ M \mid \operatorname{tr}(M) = 0 \}$. This has a natural basis of
\[
E = \begin{bmatrix} 0 & 1 \\ 0 & 0 \end{bmatrix}
\qquad
F = \begin{bmatrix} 0 & 0 \\ 1 & 0 \end{bmatrix}
\qquad
H = \begin{bmatrix} 1 & 0 \\ 0 & -1 \end{bmatrix}
\]
︡f63e3338-648f-4a68-82e6-3836479aadf5︡{"done":true,"html":"In order to look at crystals, we first start with the Lie algebra $\\mathfrak{sl}_2 = \\{ M \\mid \\operatorname{tr}(M) = 0 \\}$. This has a natural basis of\n\\[\nE = \\begin{bmatrix} 0 & 1 \\\\ 0 & 0 \\end{bmatrix}\n\\qquad\nF = \\begin{bmatrix} 0 & 0 \\\\ 1 & 0 \\end{bmatrix}\n\\qquad\nH = \\begin{bmatrix} 1 & 0 \\\\ 0 & -1 \\end{bmatrix}\n\\]"}
︠b78aa8a9-dfb9-4fb6-b917-80cc82af3ebfs︠
%auto
from sage.structure.element_wrapper import ElementWrapper
from sage.structure.parent import Parent
from sage.structure.unique_representation import UniqueRepresentation
from sage.categories.modules import Modules

class sl2(Parent, UniqueRepresentation):
    def __init__(self, R):
        Parent.__init__(self, R, category=Modules(R).WithBasis().FiniteDimensional())

    def _repr_(self):
        return "sl2 over {}".format(self.base_ring())

    @cached_method
    def basis(self):
        MS = MatrixSpace(self.base_ring(), 2)
        return Family({'E': self.element_class(self, MS([[0,1],[0,0]])),
                       'F': self.element_class(self, MS([[0,0],[1,0]])),
                       'H': self.element_class(self, MS([[1,0],[0,-1]]))})

    class Element(ElementWrapper):
        def _add_(self, rhs):
            return type(self)(self.parent(), self.value + rhs.value)

        def _sub_(self, rhs):
            return type(self)(self.parent(), self.value - rhs.value)

        def _neg_(self):
            return type(self)(self.parent(), -self.value)

        def _acted_upon_(self, scalar, self_on_left=True):
            return type(self)(self.parent(), scalar * self.value)

        def bracket(self, rhs):
            return type(self)(self.parent(), self.value * rhs.value - rhs.value * self.value)
︡458cd922-6b75-471d-b2a0-eaf07f9a6615︡{"done":true}︡
︠6d40a0d4-41d1-42e0-a741-8c70bc4e5351s︠
L = sl2(QQ)
L
︡41dabadf-bfa1-4c12-aed1-8b07b1ab211c︡{"stdout":"sl2 over Rational Field\n"}︡{"done":true}︡
︠e6854f6d-27a5-4b87-9527-30e3f59f9c03s︠
L.basis()
︡8b52a3d4-1a39-4e11-8814-4c27e1cf850e︡{"stdout":"Finite family {'H': [ 1  0]\n[ 0 -1], 'E': [0 1]\n[0 0], 'F': [0 0]\n[1 0]}\n"}︡{"done":true}︡
︠8ec64c08-c588-4155-a797-06484949fcc3s︠
B = L.basis()
E,F,H = B['E'], B['F'], B['H']
E
︡88c0526d-59c7-47f9-95d0-f7e8e3e5f71b︡{"stdout":"[0 1]\n[0 0]\n"}︡{"done":true}︡
︠0a7532c3-9fde-40d6-b55c-7699052eac09s︠
H
︡a449428a-1627-4105-ad13-fb269f4c3abd︡{"stdout":"[ 1  0]\n[ 0 -1]\n"}︡{"done":true}︡
︠5a8b2dc8-1961-45d9-9f95-d433fb2e96dcs︠
E + H
︡c49f4904-6e37-478f-9ff6-91c9d40b303f︡{"stdout":"[ 1  1]\n[ 0 -1]\n"}︡{"done":true}︡
︠ebddacc7-9912-4e3a-9263-cd5c7c1a0025s︠
H.bracket(E)
︡678bf8ea-7c48-4275-bc45-b45bb482a6e5︡{"stdout":"[0 2]\n[0 0]\n"}︡{"done":true}︡
︠1eb531c4-4b98-4833-ab13-048de3bca8dfs︠
H.bracket(F)
︡3581e52c-79de-4b60-be03-9827d028fda9︡{"stdout":"[ 0  0]\n[-2  0]\n"}︡{"done":true}︡
︠c0df2441-11b3-437a-9b64-e565497be9ccs︠
E.bracket(F)
︡cfc49623-b2b9-41d5-bbfe-9e8cfa078d67︡{"stdout":"[ 1  0]\n[ 0 -1]\n"}︡{"done":true}︡
︠351a050d-f2b8-4b82-8196-e69f68fc9ca3s︠
E * 2
︡6094cfcb-574b-4df1-bdc9-9b777ee07902︡{"stdout":"[0 2]\n[0 0]\n"}︡{"done":true}︡
︠978c7642-dcc9-49fa-a546-18ed7f63c37ds︠
3/2 * H
︡fbc22201-96f1-4f93-906e-da8c28f9bc08︡{"stdout":"[ 3/2    0]\n[   0 -3/2]\n"}︡{"done":true}︡
︠10d14560-d3e1-42c0-9eb2-c5725517f183s︠
F - E
︡217454be-a799-49ee-aeb1-0e709228818a︡{"stdout":"[ 0 -1]\n[ 1  0]\n"}︡{"done":true}︡
︠5df338b0-4e97-4d00-9dc5-ea91fac215bbi︠
%html
More generally, for $\mathfrak{sl}_n$, there are three types of basis elements: upper triangular, lower triangular, and diagonal matrices. These are generated, as a Lie algebra, by $E_i$, $F_i$, and $H_i$ respectively. The upper and lower half are "eigenvectors" for the <em>Cartan subalgebra</em> $\mathfrak{h} := \langle H_1, \dots, H_{n-1} \rangle$, i.e., $[H, E_i] = \alpha_i(H) E_i$ and $[H, F_i] = -\alpha_i(H) F_i$. The eigenvalues, considered as elements in $\mathfrak{h}^*$, are called <em>roots</em> and form a <em>root system</em> with $\{\alpha_i\}_{i \in I}$ being the simple roots.
︡0e4c595e-a5ca-4e1c-8554-dc42f74733a2︡{"done":true,"html":"More generally, for $\\mathfrak{sl}_n$, there are three types of basis elements: upper triangular, lower triangular, and diagonal matrices. These are generated, as a Lie algebra, by $E_i$, $F_i$, and $H_i$ respectively. The upper and lower half are \"eigenvectors\" for the <em>Cartan subalgebra</em> $\\mathfrak{h} := \\langle H_1, \\dots, H_{n-1} \\rangle$, i.e., $[H, E_i] = \\alpha_i(H) E_i$ and $[H, F_i] = -\\alpha_i(H) F_i$. The eigenvalues, considered as elements in $\\mathfrak{h}^*$, are called <em>roots</em> and form a <em>root system</em> with $\\{\\alpha_i\\}_{i \\in I}$ being the simple roots."}
︠98aed1eb-5754-4408-9adb-fe6fecbfec2ds︠
A = RootSystem(['A',2]).ambient_space()
A.plot(roots='all')
︡80c46173-9070-4a4a-9bba-89cd8bb7a1d8︡{"file":{"filename":"/projects/78ab9ed6-f958-4f9e-9c71-bcd84f7cb06c/.sage/temp/compute3-us/20908/tmp_6QRRaZ.svg","show":true,"text":null,"uuid":"7a052c98-61e9-442a-957b-8166d5d5b52b"},"once":false}︡{"html":"<div align='center'></div>"}︡{"done":true}︡
︠ef06c319-ba17-46b1-80a5-70bae6f123ffs︠
RootSystem(['B',2]).ambient_space().plot(roots='all')
︡b1742005-1fd8-4f06-abda-1df79d0014a3︡{"file":{"filename":"/projects/78ab9ed6-f958-4f9e-9c71-bcd84f7cb06c/.sage/temp/compute3-us/22310/tmp_tPtvy2.svg","show":true,"text":null,"uuid":"c99519d4-2843-40c6-a84e-754bae178a98"},"once":false}︡{"html":"<div align='center'></div>"}︡{"done":true}︡
︠7d101578-d3ed-44d6-b854-e08af431ac61s︠
RootSystem(['C',2]).ambient_space().plot(roots='all')
︡2cd8c14e-d640-49a2-bebd-aece5675710b︡{"file":{"filename":"/projects/78ab9ed6-f958-4f9e-9c71-bcd84f7cb06c/.sage/temp/compute3-us/22310/tmp_yhMKZd.svg","show":true,"text":null,"uuid":"9e3ee899-57cc-4f5d-8ea4-5770c3e8ea4f"},"once":false}︡{"html":"<div align='center'></div>"}︡{"done":true}︡
︠869c72b2-4c46-428b-942e-58f30d680b8cs︠
RootSystem(['G',2]).ambient_space().plot(roots='all')
︡26c20171-b0a5-4687-b495-8954a00a8ac3︡{"file":{"filename":"/projects/78ab9ed6-f958-4f9e-9c71-bcd84f7cb06c/.sage/temp/compute3-us/22310/tmp_29AKtH.svg","show":true,"text":null,"uuid":"2098bd4e-3511-469e-aba3-52df5c4cce87"},"once":false}︡{"html":"<div align='center'></div>"}︡{"done":true}︡
︠05203d69-3253-48ea-8c6c-b23277d626c9i︠
%html
We construct representations of $\mathfrak{sl}_n$ by restricting to copies of $\mathfrak{sl}_2$, where we understand the irreducible finite-dimensional representations, and piecing them together. Specifically, the irreducible representations $V_m$ for $m \in \ZZ_{>0}$ have a basis $\{v_{-m}, v_{-m+2}, \dotsc, v_{m-2}, v_{m}\}$ with actions
\begin{align*}
E v_k & = \left(\frac{k+m}{2} + 1\right) v_{k+2},
\\ F v_k & = \left(\frac{m-k}{2} + 1\right) v_{k-2},
\\ H v_k & = k v_k.
\end{align*}
︡41bc2481-9806-4a4c-918c-ab956f0b8186︡{"done":true,"html":"We construct representations of $\\mathfrak{sl}_n$ by restricting to copies of $\\mathfrak{sl}_2$, where we understand the irreducible finite-dimensional representations, and piecing them together. Specifically, the irreducible representations $V_m$ for $m \\in \\ZZ_{>0}$ have a basis $\\{v_{-m}, v_{-m+2}, \\dotsc, v_{m-2}, v_{m}\\}$ with actions\n\\begin{align*}\nE v_k & = \\left(\\frac{k+m}{2} + 1\\right) v_{k+2},\n\\\\ F v_k & = \\left(\\frac{m-k}{2} + 1\\right) v_{k-2},\n\\\\ H v_k & = k v_k.\n\\end{align*}"}
︠12725003-7184-4ffd-a6cf-336a953c5f5as︠
class sl2IrreducibleRepresentation(CombinatorialFreeModule):
    def __init__(self, R, m):
        self._m = m
        CombinatorialFreeModule.__init__(self, R, range(-m, m+1, 2), prefix='v')

    def _repr_(self):
        return "Irreducible representation of highest weight {} of sl2 over {}".format(self._m, self.base_ring())

    def highest_weight_vector(self):
        return self.basis()[self._m]

    class Element(CombinatorialFreeModule.Element):
        def E(self):
            P = self.parent()
            d = self.monomial_coefficients(copy=False)
            return P._from_dict({k+2: c * (k+P._m+2) // 2 for k,c in d.items() if k < P._m})

        def F(self):
            P = self.parent()
            d = self.monomial_coefficients(copy=False)
            return P._from_dict({k-2: c * (P._m-k+2) // 2 for k,c in d.items() if k > -P._m})

        def H(self):
            d = self.monomial_coefficients(copy=False)
            return self.parent()._from_dict({k: c*k for k,c in d.items()})
︡72165254-248f-4d30-b270-671cfb0df1a6︡{"done":true}︡
︠e14b79dd-7c9a-4f16-9fc0-1a4b93e10e50s︠
V = sl2IrreducibleRepresentation(QQ, 1)
v = V.highest_weight_vector()
v
︡6e122d0b-3661-40c8-98f6-d77569784f36︡{"stdout":"v[1]\n"}︡{"done":true}︡
︠fa7c1cb5-e239-4764-8063-4c7c0e34ab7bs︠
v.E()
︡a26d232e-31ff-4847-94ef-97c190783b13︡{"stdout":"0\n"}︡{"done":true}︡
︠7ca6ba5c-61bd-4518-b56c-9e68a612441as︠
vm = v.F()
vm
︡0da11f3e-58eb-4551-a353-f13993a76eef︡{"stdout":"v[-1]\n"}︡{"done":true}︡
︠11373b13-8c69-4c4b-9065-ccaf2deee4a8s︠
vm.F()
︡b4344f9b-29a3-4529-9b36-7cb48df32560︡{"stdout":"0\n"}︡{"done":true}︡
︠90280713-1da0-4b81-9c9c-db685baf0413s︠
vm.E()
︡22702fa9-0b62-486d-81df-f3a136be435d︡{"stdout":"v[1]\n"}︡{"done":true}︡
︠83d84fae-de96-4175-96ac-f1bdc83d8839s︠
v.H()
︡24688634-0ada-474b-ac75-ec66a0a7bc0a︡{"stdout":"v[1]\n"}︡{"done":true}︡
︠c642b6b7-ae53-4048-8615-b367c73f2da8s︠
vm.H()
︡b012f82d-9be9-4efe-8acd-164a428afcff︡{"stdout":"-v[-1]\n"}︡{"done":true}︡
︠1da20337-7ceb-470a-9358-ba7b0a8daddas︠
vm.F().E() - vm.E().F()
︡7325bf2a-01eb-4cab-8c52-edeed5006037︡{"stdout":"-v[-1]\n"}︡{"done":true}︡
︠6bbb2b9f-1fad-4909-9dad-3400c1a95182s︠
V = sl2IrreducibleRepresentation(QQ, 3)
V.basis()
︡cb7e59e8-1e23-4a32-b2d6-70386436ef00︡{"stdout":"Finite family {1: v[1], 3: v[3], -3: v[-3], -1: v[-1]}\n"}︡{"done":true}︡
︠99612413-721b-43b0-8fdf-b28425573ed6s︠
all(v.F().E() - v.E().F() == v.H() for v in V.basis())
︡e3239629-2de6-47c0-9765-b71ff7a7ea85︡{"stdout":"True\n"}︡{"done":true}︡
︠159d38cf-6a2d-4d1c-89e1-a838b28a051ei︠
%html
<p>
By ignoring these constants for the action of $E_i$ and $F_i$, we can encode representation in an edge colored, weighted directed graph as follows. We write $F_i b = \alpha b'$ as an arrow $b \xrightarrow[\hspace{20pt}]{i} b'$, where we must have $E_i b' = \alpha' b$ by the Lie algebra relations. We encode the action of $H_i$ by the weight. The resulting graph is called a <i>crystal graph</i>. Specifically, the weight must satisfy $\operatorname{wt}(f_i b) = \operatorname{wt}(b) - \alpha_i$.
</p>

<p>
For $\mathfrak{sl}_n$, the irreducible representations are indexed by partitions $\lambda$ (of height at most $n$), and the basis is given by semistandard tableaux. To determine the action of $e_i$ and $f_i$, we read a tableau top-to-bottom, right-to-left and
<ul>
<li>add any $i$ as a $+$;</li>
<li>add any $i+1$ as a $-$;</li>
<li>cancel any $+-$ pair (in that order);</li>
<li>change the $i$ corresponding to the leftmost remaining $+$ to an $i+1$.</li>
</ul>
A slight generalization of this is the tensor product rule, where we essentially only care about the dominant term. (Formally, we take a $q$-deformation of the universal enveloping algebra, then look at a Kazhdan-Lusztig like basis and take $q \to 0$.)

<p>
<b>Warning:</b> Due to Sage's tensor product convention, we reverse the order of the reading word and pairing.
</p>
</p>
︡6368117f-152c-457f-abac-1d731390e7f5︡{"done":true,"html":"<p>\nBy ignoring these constants for the action of $E_i$ and $F_i$, we can encode representation in an edge colored, weighted directed graph as follows. We write $F_i b = \\alpha b'$ as an arrow $b \\xrightarrow[\\hspace{20pt}]{i} b'$, where we must have $E_i b' = \\alpha' b$ by the Lie algebra relations. We encode the action of $H_i$ by the weight. The resulting graph is called a <i>crystal graph</i>. Specifically, the weight must satisfy $\\operatorname{wt}(f_i b) = \\operatorname{wt}(b) - \\alpha_i$.\n</p>\n\n<p>\nFor $\\mathfrak{sl}_n$, the irreducible representations are indexed by partitions $\\lambda$ (of height at most $n$), and the basis is given by semistandard tableaux. To determine the action of $e_i$ and $f_i$, we read a tableau top-to-bottom, right-to-left and\n<ul>\n<li>add any $i$ as a $+$;</li>\n<li>add any $i+1$ as a $-$;</li>\n<li>cancel any $+-$ pair (in that order);</li>\n<li>change the $i$ corresponding to the leftmost remaining $+$ to an $i+1$.</li>\n</ul>\nA slight generalization of this is the tensor product rule, where we essentially only care about the dominant term. (Formally, we take a $q$-deformation of the universal enveloping algebra, then look at a Kazhdan-Lusztig like basis and take $q \\to 0$.)\n\n<p>\n<b>Warning:</b> Due to Sage's tensor product convention, we reverse the order of the reading word and pairing.\n</p>\n</p>"}
︠3cc6d49b-2e20-49a6-aac8-e2ad2a41ba16︠
C = crystals.Tableaux(['A',2], shape=[1])
latex.eval(latex(C))
︡f113f224-a4c9-4cf1-b913-1407c0427811︡{"file":{"filename":"/tmp/tmp_XQP5N.png","show":true,"text":null,"uuid":"b4cef046-8129-42ee-9835-a0ec6a749bb3"},"once":false}︡{"stdout":"''"}︡{"stdout":"\n"}︡{"done":true}︡
︠fe3f6406-3500-45bb-987a-e88e879dc501︠
T = tensor([C, C, C])
latex.eval(latex(T))
︡0d2950fc-39dd-4e31-91fc-b6be30d91c9d︡{"file":{"filename":"/tmp/tmpuyKlqV.png","show":true,"text":null,"uuid":"a5d21c13-45a0-4968-a952-8d96e49b3cbc"},"once":false}︡{"stdout":"''"}︡{"stdout":"\n"}︡{"done":true}︡
︠36c2cdd2-1814-421f-b7d3-b23354529d82︠
mg = C.highest_weight_vector()
x = T(mg.f(1), mg, mg)
x
︡36ee3ccf-fcbe-46ba-92da-4aff09fcc080︡{"stdout":"[[[2]], [[1]], [[1]]]\n"}︡{"done":true}︡
︠eeeec21b-cba6-415c-8732-3728db9eca14︠
S = x.subcrystal()
S
︡df342812-d408-4afb-92a3-cbe69e230f48︡{"stdout":"Subcrystal of Full tensor product of the crystals [The crystal of tableaux of type ['A', 2] and shape(s) [[1]], The crystal of tableaux of type ['A', 2] and shape(s) [[1]], The crystal of tableaux of type ['A', 2] and shape(s) [[1]]]\n"}︡{"done":true}︡
︠3f024f8a-c2ae-4aff-8bda-7c8b400c42a2︠
T21 = crystals.Tableaux(['A',2], shape=[2,1])
D = crystals.DirectSum([T21, S])
latex.eval(latex(D))
︡b052a555-55c4-455e-8bc0-4162ed5d455f︡{"file":{"filename":"/tmp/tmpZbw0Gn.png","show":true,"text":null,"uuid":"51b9639c-785a-4d1c-97c3-6e27bb331d05"},"once":false}︡{"stdout":"''"}︡{"stdout":"\n"}︡{"done":true}︡
︠2e4cd440-a2c9-4c75-9bf0-a5807694da80i︠
%html
The weight is given by $\epsilon_i$ coefficient counts the number of $i$'s in the tableaux. This is exactly analogous to the definition of Schur functions (which are the characters of the irreducible highest weight representations of $\mathfrak{sl}_n$). To connect this back to the action of $\mathfrak{h}$, $\epsilon_i$ gives the coefficient on the $i$-diagonal entry. Hence, we can plot the crystal in the ambient space.
︡c982498f-b26c-4f6a-ae89-7bbb7aba4384︡{"done":true,"html":"The weight is given by $\\epsilon_i$ coefficient counts the number of $i$'s in the tableaux. This is exactly analogous to the definition of Schur functions (which are the characters of the irreducible highest weight representations of $\\mathfrak{sl}_n$). To connect this back to the action of $\\mathfrak{h}$, $\\epsilon_i$ gives the coefficient on the $i$-diagonal entry. Hence, we can plot the crystal in the ambient space."}
︠35623e5c-5658-4bf9-a05e-a5fdb80c91d3︠
A = RootSystem(['A',2,1]).ambient_space()
p = A.plot_alcoves() + A.plot_crystal(T21, plot_labels='circles')
p.axes(False)
p
︡047c8c68-09de-40be-8f61-e8c3938a68a9︡{"file":{"filename":"/projects/681dd0ac-80e2-4cfd-a5ae-9c6bef57ebee/.sage/temp/compute1-us/14463/tmp_zzJLhx.svg","show":true,"text":null,"uuid":"dfdfe9a4-99ca-401d-b634-cd3c1f40014f"},"once":false}︡{"html":"<div align='center'></div>"}︡{"done":true}︡
︠dc995540-4d87-4762-b9fe-734b6949bcfd︠
A = RootSystem(['B',3]).ambient_space()
B = crystals.Tableaux(['B',3], shape=[1,1])
A.plot_fundamental_weights() + A.plot_crystal(B, edge_labels=True, plot_labels='circles')
︡804f30f2-a862-43e0-9f0b-36ada62301e0︡{"file":{"filename":"cc3b58ab-d63f-4ed0-802d-e0633a4e155a.sage3d","uuid":"cc3b58ab-d63f-4ed0-802d-e0633a4e155a"}}︡{"html":"<div align='center'></div>"}︡{"done":true}︡
︠0ddedefa-7bba-4cad-a48e-b04f0e7d7bcei︠
%html
<p>
While tableaux are nice, they don't work well beyond the classical types ($A, B, C, D$) and $G_2$. This is because it requires knowledge of the representation of $B(\Lambda_1)$, and it uses the semi-simplicity of $\mathfrak{g}$. However, we want something that works for <i>Kac-Moody Lie algebras</i>, the generalization of $\mathfrak{sl}_n$, $\mathfrak{so}_{2n+1}$, $\mathfrak{sp}_{2n}$, $\mathfrak{so}_{2n}$ which includes all of the affine Lie algebras. Moreover, we want a model that is <i>uniform</i>, that is only depends on the underlying root system which defines $\mathfrak{g}$.
</p>

<p>
The first known such model is the <i>Littelmann path model</i>. The elements (vertices) are given by paths $\pi \colon [0,1] \to \mathfrak{h}^*$ in the weight space, and the crystal operators are given by partial reflections of the path.
</p>
︡6aee0570-4ebc-4809-bd1e-42b1f3951fdb︡{"done":true,"html":"<p>\nWhile tableaux are nice, they don't work well beyond the classical types ($A, B, C, D$) and $G_2$. This is because it requires knowledge of the representation of $B(\\Lambda_1)$, and it uses the semi-simplicity of $\\mathfrak{g}$. However, we want something that works for <i>Kac-Moody Lie algebras</i>, the generalization of $\\mathfrak{sl}_n$, $\\mathfrak{so}_{2n+1}$, $\\mathfrak{sp}_{2n}$, $\\mathfrak{so}_{2n}$ which includes all of the affine Lie algebras. Moreover, we want a model that is <i>uniform</i>, that is only depends on the underlying root system which defines $\\mathfrak{g}$.\n</p>\n\n<p>\nThe first known such model is the <i>Littelmann path model</i>. The elements (vertices) are given by paths $\\pi \\colon [0,1] \\to \\mathfrak{h}^*$ in the weight space, and the crystal operators are given by partial reflections of the path.\n</p>"}
︠c8c4705b-0a52-4f22-8324-83180f74033d︠
La = RootSystem(['C',2]).weight_space().fundamental_weights()
B = crystals.LSPaths(3*La[1]+La[2])
mg = B.highest_weight_vector()
A = RootSystem(['C',2,1]).ambient_space()
elts = [mg, mg.f(1), mg.f(1).f(1), mg.f_string([1,1,1]), mg.f(2), mg.f(2).f(1)]
p = A.plot(reflection_hyperplanes=False, bounding_box=4) + A.plot_ls_paths(elts)
p.axes(False)
p
︡075bb2e8-5e9b-4a30-9ee8-9c288c28de7f︡{"file":{"filename":"/projects/681dd0ac-80e2-4cfd-a5ae-9c6bef57ebee/.sage/temp/compute1-us/14463/tmp_u9aZKZ.svg","show":true,"text":null,"uuid":"7cc8572a-028a-4f27-9bae-a6eb6bd492cd"},"once":false}︡{"html":"<div align='center'></div>"}︡{"done":true}︡
︠02a434cf-bfbc-4ae0-9fa5-b1452bcd24c5i︠
%html
There is a discrete counterpart to the Littelmann path model called the alcove path model. In this model, elements are given by alcove walks, reflect the Bruhat graph, and are related to the equivariant K-theory of flag varities. The crystal operators are given by folding the alcove walk.
︡ff36ea59-07fc-44b3-a31e-d20912ac8a01︡{"done":true,"html":"There is a discrete counterpart to the Littelmann path model called the alcove path model. In this model, elements are given by alcove walks, reflect the Bruhat graph, and are related to the equivariant K-theory of flag varities. The crystal operators are given by folding the alcove walk."}
︠fa5f7bcf-61e6-40ad-9dcf-1fbbdd7bcc73︠
B = crystals.AlcovePaths(['C',2], [3,1])
mg = B.module_generators[0]
A = RootSystem(['C',2,1]).ambient_space()
mg.plot()
︡2ae71a74-3619-4411-ade7-a0d019ef2abc︡{"file":{"filename":"/projects/681dd0ac-80e2-4cfd-a5ae-9c6bef57ebee/.sage/temp/compute1-us/26516/tmp_3Gl9LT.svg","show":true,"text":null,"uuid":"1a404c76-1ff5-4fed-a47d-fec9fde962c0"},"once":false}︡{"html":"<div align='center'></div>"}︡{"done":true}︡
︠1d61f2ef-c304-4f92-9ad5-26727978d966︠
mg.f(1).plot()
︡ac913386-3d59-406d-b0e1-ac98f5078eaf︡{"file":{"filename":"/projects/681dd0ac-80e2-4cfd-a5ae-9c6bef57ebee/.sage/temp/compute1-us/26516/tmp_FhUvlH.svg","show":true,"text":null,"uuid":"e3b2d15f-e973-4aa9-8087-f5e7cf296845"},"once":false}︡{"html":"<div align='center'></div>"}︡{"done":true}︡
︠36f9f61d-9425-43bb-aebc-2fb0e3aee85f︠
mg.f(2).plot(bounding_box=4)
︡87bfec23-1236-4755-8a1c-c61a7e60b3b0︡{"file":{"filename":"/projects/681dd0ac-80e2-4cfd-a5ae-9c6bef57ebee/.sage/temp/compute1-us/26516/tmp_j0NRgD.svg","show":true,"text":null,"uuid":"22383bb4-2a40-4d4c-83db-ecbf95773a21"},"once":false}︡{"html":"<div align='center'></div>"}︡{"done":true}︡
︠6cb03f7a-3469-4be2-b3b8-492333e536a0︠
mg.f(2).f(1).plot()
︡9e2f2633-4ea2-4444-8963-cff7dfb83b72︡{"file":{"filename":"/projects/681dd0ac-80e2-4cfd-a5ae-9c6bef57ebee/.sage/temp/compute1-us/26516/tmp_Cce22v.svg","show":true,"text":null,"uuid":"3308d2af-6a7b-49a1-a59f-127edcfeb771"},"once":false}︡{"html":"<div align='center'></div>"}︡{"done":true}︡
︠7c1e9b27-a978-4f0e-9daf-649e8889bfe7i︠
%html
Another model for crystals is given by <i>Nakajima monomials</i>, whsoe elements are Laurent monomials in $\{Y_{i,k} \mid i \in I, k \in \ZZ\}$ and the crystal operators are multiplication by certain monomials $A_{i,k}$. In particular, the weight of $Y_{i,k}$ is $\Lambda_i$, and so the $A_{i,k}$ are a modification of the simple root $\alpha_i$.
︡7fcdd193-07a4-49eb-97cc-b4d7ea6c18b9︡{"done":true,"html":"Another model for crystals is given by <i>Nakajima monomials</i>, whsoe elements are Laurent monomials in $\\{Y_{i,k} \\mid i \\in I, k \\in \\ZZ\\}$ and the crystal operators are multiplication by certain monomials $A_{i,k}$. In particular, the weight of $Y_{i,k}$ is $\\Lambda_i$, and so the $A_{i,k}$ are a modification of the simple root $\\alpha_i$."}
︠243e3c8a-e243-41f5-9bce-c0e26cd9b8ee︠
La = RootSystem(['A',2,1]).weight_lattice(extended=True).fundamental_weights()
M = crystals.NakajimaMonomials(['A',2,1], La[0])
S = M.subcrystal(max_depth=4)
G = S.digraph()
G.set_latex_options(edge_options=None)
latex.eval(latex(G))
︡0a191f71-b8f4-4d27-8282-567b42a5ef98︡{"file":{"filename":"/tmp/tmpal2Mnt.png","show":true,"text":null,"uuid":"9e626399-81dc-4cbf-b76b-1a10d49487f8"},"once":false}︡{"stdout":"''"}︡{"stdout":"\n"}︡{"done":true}︡
︠833f239b-1c98-44e3-b929-92ec9c0a26ebi︠
%html
Note that highest weight crystals/representations are indexed by elements in the dominant weight lattice $P^+ := \span_{\ZZ_{>0}} \{\Lambda_i\}_{i \in I}$. We demonstrate one last model, <i>rigged configurations</i>, which was given by Salisbury and S. generalizing the work of Schilling. Elements are given by partitions with integers (called riggings) attached to each row, one for each node of the Dynkin diagram. The crystal operators $f_i$ (resp. $e_i$) adds (resp. removes) a box from the partition $\nu^{(i)}$.
︡e087535c-7d91-4547-b470-72132e6ba92f︡{"done":true,"html":"Note that highest weight crystals/representations are indexed by elements in the dominant weight lattice $P^+ := \\span_{\\ZZ_{>0}} \\{\\Lambda_i\\}_{i \\in I}$. We demonstrate one last model, <i>rigged configurations</i>, which was given by Salisbury and S. generalizing the work of Schilling. Elements are given by partitions with integers (called riggings) attached to each row, one for each node of the Dynkin diagram. The crystal operators $f_i$ (resp. $e_i$) adds (resp. removes) a box from the partition $\\nu^{(i)}$."}
︠df579f26-7302-4a5a-85d6-56a6928d7abc︠
D = DynkinDiagram()
for i in range(4):
    for j in range(i+1,4):
        D.add_edge(i,j)
La = RootSystem(D.cartan_matrix()).weight_lattice().fundamental_weights()
RC = crystals.RiggedConfigurations(La[0])
S = RC.subcrystal(max_depth=2)
G = S.digraph()
G.set_latex_options(edge_options=None)
latex.eval(latex(G))
︡398ae59a-a867-44df-ade4-032a01728f2f︡{"file":{"filename":"/tmp/tmp4hK3dx.png","show":true,"text":null,"uuid":"9352c8be-c28b-4159-9729-6be41bbe4738"},"once":false}︡{"stdout":"''"}︡{"stdout":"\n"}︡{"done":true}︡
︠f88824e5-6c72-4f2b-ab03-41f7a3d67d8bi︠
%html
<p>
<b>Open problem:</b> Find an (explicit) combinatorial bijection between LS/Littelmann paths, Nakajima monomials, and rigged configurations.
</p>

<p>
One particular aspect is that $B(\lambda)$, for any fixed $\lambda \in P^+$, is unique, and there is a unique crystal isomorphism by mapping the highest weight element to the highest weight element.
</p>
︡fed7701a-a283-45d7-aeba-f5859f9bfb20︡{"done":true,"html":"<p>\n<b>Open problem:</b> Find an (explicit) combinatorial bijection between LS/Littelmann paths, Nakajima monomials, and rigged configurations.\n</p>\n\n<p>\nOne particular aspect is that $B(\\lambda)$, for any fixed $\\lambda \\in P^+$, is unique, and there is a unique crystal isomorphism by mapping the highest weight element to the highest weight element.\n</p>"}
︠d10cc2ee-6525-4744-8b0d-5e4b629de708︠
La = RootSystem(['C',2]).weight_lattice().fundamental_weights()
M = crystals.NakajimaMonomials(['C',2], La[1]+La[2])
RC = crystals.RiggedConfigurations(La[1]+La[2])
phi = M.crystal_morphism({M.highest_weight_vector(): RC.highest_weight_vector()})
for m in M:
    A = ascii_art(phi(m))
    A._baseline = 0
    A = ascii_art(m) + ascii_art('   |-->   ') + A
    print(A)
    print("")
︡3a9f43fd-e93b-48a0-8609-dd0cb7d4abfb︡{"stdout":"Y(1,0) Y(2,0)    |-->   (/)  (/)\n\nY(1,1)^-1 Y(2,0)^2    |-->   -1[ ]-1  (/)\n\nY(1,1) Y(2,0) Y(2,1)^-1    |-->   0[ ]0  0[ ]-1\n\nY(1,2)^-1 Y(2,0)    |-->   -1[ ][ ]-1  1[ ]0\n\nY(1,1)^3 Y(2,1)^-2    |-->   0[ ]0  -2[ ][ ]-2\n\nY(1,1)^2 Y(1,2)^-1 Y(2,1)^-1    |-->   -1[ ][ ]-1  -1[ ][ ]-1\n\nY(1,1) Y(1,2)^-2    |-->   -2[ ][ ][ ]-2  0[ ][ ]0\n\nY(1,2)^-3 Y(2,1)    |-->   -3[ ][ ][ ][ ]-3  1[ ][ ]1\n\nY(1,0) Y(1,1)^2 Y(2,1)^-1    |-->   (/)  -1[ ]-1\n\nY(1,0) Y(1,1) Y(1,2)^-1    |-->   0[ ]-1  0[ ]0\n\nY(1,0) Y(1,2)^-2 Y(2,1)    |-->   -1[ ][ ]-2  1[ ]1\n\nY(1,1)^-1 Y(1,2)^-2 Y(2,0) Y(2,1)    |-->   -3[ ][ ][ ]-3  1[ ]1\n\n                           1[ ][ ]0  -1[ ]-1\nY(1,0) Y(2,2)^-1    |-->             -1[ ]-1\n\n                                     -1[ ][ ][ ]-1  -1[ ]-1\nY(1,1)^-1 Y(2,0) Y(2,2)^-1    |-->                  -1[ ]-1\n\n                                     0[ ][ ][ ]0  -2[ ][ ]-2\nY(1,1) Y(2,1)^-1 Y(2,2)^-1    |-->                -1[ ]-1\n\n                              -1[ ][ ][ ][ ]-1  -1[ ][ ]-1\nY(1,2)^-1 Y(2,2)^-1    |-->                     -1[ ]-1\n\n"}︡{"done":true}︡
︠a5f77d4d-0c5c-4ab9-8219-d38833989876i︠
%html
One particularly nice property of rigged configurations is that they all "nest" inside each other, i.e., $B(\lambda) \subseteq B(\mu)$ if $\lambda \leq \mu$. This allows us to construct the direct limit $B(\infty)$, which is the crystal for the lower half of the quantum group (or equivalently, the universal enveloping algebra).
︡15a8c384-bd32-48ab-8bc3-95cfbbe0bf6f︡{"done":true,"html":"One particularly nice property of rigged configurations is that they all \"nest\" inside each other, i.e., $B(\\lambda) \\subseteq B(\\mu)$ if $\\lambda \\leq \\mu$. This allows us to construct the direct limit $B(\\infty)$, which is the crystal for the lower half of the quantum group (or equivalently, the universal enveloping algebra)."}
︠c520b48d-b6a0-49c0-9976-be347cdea076︠
La = RootSystem(['C',2]).weight_lattice().fundamental_weights()
RC1 = crystals.RiggedConfigurations(['C',2], La[1])
RC2 = crystals.RiggedConfigurations(['C',2], La[2])
latex.eval(latex(RC1 + RC2))
︡565406ce-b82c-4962-8726-bf8429847a2b︡{"file":{"filename":"/tmp/tmpa7AvJ7.png","show":true,"text":null,"uuid":"8bcfae3a-6dd0-4d27-a95b-43e2a5df69f7"},"once":false}︡{"stdout":"''"}︡{"stdout":"\n"}︡{"done":true}︡
︠855480b1-57b5-4b88-bb66-2bb54a700bca︠
RC = crystals.RiggedConfigurations(['C',2], La[1]+La[2])
latex.eval(latex(RC))
︡124a51e0-81a2-4fc9-a9ff-c6e5958bd5a2︡{"file":{"filename":"/tmp/tmpe1T_bi.png","show":true,"text":null,"uuid":"fd2e9bed-b6d9-4931-82c3-54410d7fe7bd"},"once":false}︡{"stdout":"''"}︡{"stdout":"\n"}︡{"done":true}︡
︠eb92ea51-0b4a-4850-8548-a09263d6d211︠
RC = crystals.infinity.RiggedConfigurations(['C',2])
S = RC.subcrystal(max_depth=2)
latex.eval(latex(S))
︡b63da06e-ea71-4607-b3a2-3dc1b7c54272︡{"file":{"filename":"/tmp/tmp622o8F.png","show":true,"text":null,"uuid":"d07c2b99-2927-483f-9e83-02073c89063a"},"once":false}︡{"stdout":"''"}︡{"stdout":"\n"}︡{"done":true}︡
︠38db0d86-a464-4e9f-94a3-df689e7ce4a0i︠
%html
<p>
There are also models for $B(\infty)$ using (modified) Nakajima monomials, LS paths, and the polyhedral realization. There are also a number of more specialized models: (marginally large) tableaux (classical types), or those only for (affine) type $A$.
</p>

<p>
More generally, we can recover $B(\lambda)$ by tensoring with a specific one element crystal, denoted by $R_{\lambda}$. <i>Note:</i> the order of the tensor factors is important.
</p>
︡2c8b556b-a61f-41fe-b99a-1ee11ee96ed9︡{"done":true,"html":"<p>\nThere are also models for $B(\\infty)$ using (modified) Nakajima monomials, LS paths, and the polyhedral realization. There are also a number of more specialized models: (marginally large) tableaux (classical types), or those only for (affine) type $A$.\n</p>\n\n<p>\nMore generally, we can recover $B(\\lambda)$ by tensoring with a specific one element crystal, denoted by $R_{\\lambda}$. <i>Note:</i> the order of the tensor factors is important.\n</p>"}
︠49b070cd-ffd9-4461-8ab3-0d504e191762︠
La = RootSystem(['E',6]).weight_lattice().fundamental_weights()
B = crystals.infinity.NakajimaMonomials(['E',6])
R = crystals.elementary.R(['E',6], La[1])
T = tensor([R, B])
C = crystals.RiggedConfigurations(['E',6], La[1])
hw = T(R.module_generators[0], B.highest_weight_vector())
G = hw.subcrystal().digraph()
G.is_isomorphic(C.digraph(), edge_labels=True)
︡e8102d04-5941-4afd-84d1-99ab7f00d1b5︡{"stdout":"True\n"}︡{"done":true}︡
︠7d5ac142-b869-4291-8c97-0ee3600df230i︠
%html
<p>
There is also one other special class of crystals for affine types called <em>Kirillov-Reshetikhin (KR) crystals</em>, which are finite crystals denoted by $B^{r,s}$ with $r \in I$ and $s \in \ZZ_{>0}$. While there exists general (and uniform) theories for highest weight crystals, there is very little general and/or uniform understanding of KR crystals. The only known uniform constructions are for $\bigotimes_{i=1}^N B^{r_i, 1}$. Depsite this, KR crystals are known to have many wonderful properties, such as tensor products of KR crystals are connected. Moreover, (tensor products of) KR crystals are strongly related to multiple aspects of mathematical physics, such as box-ball systems (or soliton cellular automata), spin chains, and 2d solvable lattice models.
</p>

<p>
<b>Open problem:</b> Construct a uniform model for $B^{r,s}$.
</p>
︡1a4cd730-bdc9-4b79-9e05-a3592ab0c762︡{"done":true,"html":"<p>\nThere is also one other special class of crystals for affine types called <em>Kirillov-Reshetikhin (KR) crystals</em>, which are finite crystals denoted by $B^{r,s}$ with $r \\in I$ and $s \\in \\ZZ_{>0}$. While there exists general (and uniform) theories for highest weight crystals, there is very little general and/or uniform understanding of KR crystals. The only known uniform constructions are for $\\bigotimes_{i=1}^N B^{r_i, 1}$. Depsite this, KR crystals are known to have many wonderful properties, such as tensor products of KR crystals are connected. Moreover, (tensor products of) KR crystals are strongly related to multiple aspects of mathematical physics, such as box-ball systems (or soliton cellular automata), spin chains, and 2d solvable lattice models.\n</p>\n\n<p>\n<b>Open problem:</b> Construct a uniform model for $B^{r,s}$.\n</p>"}
︠bfcebe3a-0362-4079-a30f-ca944947be83︠
K = crystals.KirillovReshetikhin(['A',2,1], 2,2)
latex.eval(latex(K))
︡aa54c896-7010-4bcc-84fe-4ab276f6ecae︡{"file":{"filename":"/tmp/tmpahzMqL.png","show":true,"text":null,"uuid":"05cd3919-b4cf-4145-b724-be88e62e4f90"},"once":false}︡{"stdout":"''"}︡{"stdout":"\n"}︡{"done":true}︡
︠b2ed286b-8447-4d3f-9297-d3686ebcc301︠
K2 = crystals.KirillovReshetikhin(['A',2,1], 2,1)
latex.eval(latex(K2))
︡034ead1a-0cf6-41d6-95c1-aa56b90830d4︡{"file":{"filename":"/tmp/tmpmp9oYK.png","show":true,"text":null,"uuid":"5b924050-0d24-4974-9ea6-95883bf3c427"},"once":false}︡{"stdout":"''"}︡{"stdout":"\n"}︡{"done":true}︡
︠c5621823-265e-4e52-a550-adade336d4b9︠
T = tensor([K2, K2])
latex.eval(latex(T))
︡926f4ef2-e16d-4aaa-aac9-d58efd81196c︡{"file":{"filename":"/tmp/tmpnv2kDz.png","show":true,"text":null,"uuid":"9548e705-8072-49b4-b470-ecc019b93e10"},"once":false}︡{"stdout":"''"}︡{"stdout":"\n"}︡{"done":true}︡
︠ab58e133-303a-499a-8419-33e802c48424︠
K1 = crystals.KirillovReshetikhin(['A',2,1], 1,1)
T = tensor([K1,K1])
latex.eval(latex(T))
︡ed2f243c-bfd8-4512-b262-0a032a3762af︡{"file":{"filename":"/tmp/tmptYx3ix.png","show":true,"text":null,"uuid":"fdcd0c23-da7d-4836-938b-f8a1d02f4838"},"once":false}︡{"stdout":"''"}︡{"stdout":"\n"}︡{"done":true}︡
︠d038412d-b064-4c74-98bc-04f6eee7e1c2︠
K = crystals.KirillovReshetikhin(['A',2,1], 1,2)
latex.eval(latex(K))
︡9ba64710-19e5-47d2-b399-3abe226978fe︡{"file":{"filename":"/tmp/tmpbeN4s_.png","show":true,"text":null,"uuid":"83fa93f4-56a9-47ee-8df9-9959878a09fd"},"once":false}︡{"stdout":"''"}︡{"stdout":"\n"}︡{"done":true}︡
︠2ae87615-c353-40e7-9c2e-366f17675fd8i︠
%html
<p>
There are two important properties that characters of tensor products of KR crystals have. The first is that their characters (resp. $q$-characters) solve Q-systems (resp. T-systems). Morever, there is that there is a statistic called <em>energy</em>, such that the graded character is a specalization of Macdonald polynomials and allows one to compute Kostka polynomials. We utilize here an equivalence with a slightly different version of rigged configurations than what is used by highest weight crystals.
</p>

<p>
<b>Open problem:</b> Show such a bijection exists in all (affine) types. Bonus points for stating and/or proving it uniformly.
</p>
︡aabeadeb-c174-449f-90dc-d68aa71e052c︡{"done":true,"html":"<p>\nThere are two important properties that characters of tensor products of KR crystals have. The first is that their characters (resp. $q$-characters) solve Q-systems (resp. T-systems). Morever, there is that there is a statistic called <em>energy</em>, such that the graded character is a specalization of Macdonald polynomials and allows one to compute Kostka polynomials. We utilize here an equivalence with a slightly different version of rigged configurations than what is used by highest weight crystals.\n</p>\n\n<p>\n<b>Open problem:</b> Show such a bijection exists in all (affine) types. Bonus points for stating and/or proving it uniformly.\n</p>"}
︠261d2994-33ae-49f1-b979-81e5bcbc17e0s︠
RC = RiggedConfigurations(['A',7,1], [[3,1],[1,1],[1,1]])
RC.fermionic_formula(only_highest_weight=True)
︡0d51ceec-349c-4055-93b2-4cb9d1602477︡{"stdout":"(q^2+q)*B[Lambda[1] + Lambda[4]] + B[2*Lambda[1] + Lambda[3]] + q*B[Lambda[2] + Lambda[3]] + q^3*B[Lambda[5]]"}︡{"stdout":"\n"}︡{"done":true}︡
︠60ce82c9-c9a6-4f2a-a24d-075d1a8ed9cfs︠
t = FractionField(ZZ['t']).gen()
Sym = SymmetricFunctions(t.parent())
s = Sym.s()
h = Sym.h()
P = Sym.hall_littlewood(t).P()
︡83ad4a48-35ec-4166-aa48-5973725c2a52︡{"done":true}︡
︠a8fed6e2-61cc-4cd7-ba2b-2714f724aef0s︠
P(s[4,1])
︡564fc645-4e29-4a9b-9241-7816599ece2e︡{"stdout":"(t^9+t^8+t^7+t^6)*HLP[1, 1, 1, 1, 1] + (t^5+t^4+t^3)*HLP[2, 1, 1, 1] + (t^3+t^2)*HLP[2, 2, 1] + (t^2+t)*HLP[3, 1, 1] + t*HLP[3, 2] + HLP[4, 1]"}︡{"stdout":"\n"}︡{"done":true}︡
︠c06e1662-ac8d-494b-a32e-744154f7ae37s︠
P(s[3,1,1])
︡ff415017-2dec-4598-acdc-d3647106aa18︡{"stdout":"(t^7+t^6+2*t^5+t^4+t^3)*HLP[1, 1, 1, 1, 1] + (t^3+t^2+t)*HLP[2, 1, 1, 1] + t*HLP[2, 2, 1] + HLP[3, 1, 1]\n"}︡{"done":true}︡
︠0ea7c09e-adea-40ba-a748-b2fc292fbba8s︠
P(s[3,2])
︡3d63f86e-537d-4908-b811-5d07dbaf00d7︡{"stdout":"(t^8+t^7+t^6+t^5+t^4)*HLP[1, 1, 1, 1, 1] + (t^4+t^3+t^2)*HLP[2, 1, 1, 1] + (t^2+t)*HLP[2, 2, 1] + t*HLP[3, 1, 1] + HLP[3, 2]\n"}︡{"done":true}︡
︠f46d3210-1b6f-4dbc-8a74-75170d41a82ds︠
P(s[5])
︡f3d8ee3e-f043-495d-aee0-bd971f83c14d︡{"stdout":"t^10*HLP[1, 1, 1, 1, 1] + t^6*HLP[2, 1, 1, 1] + t^4*HLP[2, 2, 1] + t^3*HLP[3, 1, 1] + t^2*HLP[3, 2] + t*HLP[4, 1] + HLP[5]\n"}︡{"done":true}︡
︠d1ad0138-eeb7-42a7-9dc9-9a68c46265b8s︠
HL = Sym.hall_littlewood(t)
︡1c957936-bc34-42d0-95ec-ce9ea7fcbac2︡{"done":true}︡
︠c5c8f151-455c-46bd-a02c-79904f438c73s︠
Qp = HL.Qp()
︡0e8e0f15-c58d-4767-a59a-1100f2d93a58︡{"done":true}︡
︠d140db8f-7147-452a-904c-6d21c0e3806ds︠
s(Qp[3,1,1])
︡7bc94f82-ea15-40ca-8eb9-a2fbb654db63︡{"stdout":"s[3, 1, 1] + t*s[3, 2] + (t^2+t)*s[4, 1] + t^3*s[5]"}︡{"stdout":"\n"}︡{"done":true}︡
︠5bd6aba6-b735-448e-981e-8375e3fce7ef︠









